from typing import Any

from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest
from django.utils.functional import classproperty
from django_ical.views import ICalFeed

from aleksis.core.util.core_helpers import get_site_preferences


class PersonalICalFeedBase(ICalFeed):
    @property
    def product_id(self):
        lang = self.request.LANGUAGE_CODE
        return f"-//AlekSIS®//{get_site_preferences()['general__title']}//{lang}"

    link = settings.BASE_URL
    timezone = settings.TIME_ZONE
    person = None
    request = None

    def get_object(self, request: WSGIRequest, *args: Any, **kwargs: Any) -> None:
        if kwargs.get("person"):
            self.person = kwargs.pop("person")
        self.request = request
        return super().get_object(request, *args, **kwargs)

    @classproperty
    def subclasses_list(cls):
        return cls.__subclasses__()

    @classproperty
    def subclasses_dict(cls):
        return {subclass.__name__: subclass for subclass in cls.subclasses_list}

    @classproperty
    def subclass_choices(cls):
        return [(subclass.__name__, f"{subclass.title} – {subclass.description}") for subclass in cls.subclasses_list]
